
function demo_polygauss(degree,domains)

%--------------------------------------------------------------------------
% OBJECT.
%--------------------------------------------------------------------------
% This demo compares "polygauss.m" using "triangulation" option against
% the "quadrangulation" option, on several domain, testing five different
% functions.
% Max absolute errors between these functions are reported.
% By setting the "domains" variable, several domains can be considered (see
% "mydomain.m" routine).
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% EXAMPLE.
%--------------------------------------------------------------------------
%
% Setting degree=20, domains=1,
%
% >> demo_polygauss
%
%  	 ----- BY TRIANGULATION
%  	 * domaintype:   1
%  	 [POLYGON]: 6 SIDES CONVEX POLYGON :    7 VERTICES
%
%  	 ----- BY QUADRANGULATION
%  	 * domaintype:   1
%  	 [POLYGON]: 6 SIDES CONVEX POLYGON :    7 VERTICES
%
%  	 ----- BY TRIANGULATION (REFERENCE VALUE)
%  	 * domaintype:   1
%  	 [POLYGON]: 6 SIDES CONVEX POLYGON :    7 VERTICES
%
%  	 ----- FORMULA COMPARISON
%  	 * domaintype:   1
%  	 max. abs. diff.: 5.04e-07
%  	 max. abs. errs.: T: 5.10e-07 Q: 7.45e-09
%  	 max. rel. err.: 1.35e-04
%
% >>
%
%--------------------------------------------------------------------------
% NOTE.
%--------------------------------------------------------------------------
%
% Here:
%
% "max. abs. diff." is the maximum absolute difference between the
% results achieved by the two methods on 5 tests functions.
%
% "max. abs. errs." is the maximum absolute difference between the
% results achieved by the two methods on 5 tests functions and a reference
% result by higher order rule. Notice that T stands for triang., Q for
% quadrang.
%
%--------------------------------------------------------------------------

if nargin < 1
    degree=30;
end

if nargin < 2
    % domains=[1:4 6:8 10:14];
    % domains=[15:28];
    domains=16;
end

%-------------- MAIN PROGRAM STARTS HERE ----------------------------------

doplot=1; % 0: no plot. 1: plot.

fprintf('\n \n \t ----- BY TRIANGULATION');

I=zeros(length(domains),5);

for domaintype=1:length(domains)
    fprintf('\n \t * domaintype: %3.0f',domains(domaintype));
    polygon_sides=mydomain(domains(domaintype));
    xywt=polygauss(polygon_sides,degree,...
        'triangulation');
    
    int=inpolygon(xywt(:,1),xywt(:,2),...
        polygon_sides(:,1),polygon_sides(:,2));
    s=min(int);
    if s== 0
        ss='inpolygon: ko';
    else
        ss='inpolygon: ok';
    end
    
    
    fprintf('\n \t >> '); disp(ss);
    xt=xywt(:,1); yt=xywt(:,2); wt=xywt(:,3);
    for ftype=1:5
        f=myfun(ftype);
        fxy=feval(f,xt,yt);
        I(domaintype,ftype)=wt'*fxy;
    end
end

fprintf('\n \n \t ----- BY QUADRANGULATION');

J=zeros(length(domains),5);

for domaintype=1:length(domains)
    fprintf('\n \t * domaintype: %3.0f',domains(domaintype));
    polygon_sides=mydomain(domains(domaintype));
    newroutine=0;
    
    xywq=polygauss(polygon_sides,degree,...
        'quadrangulation_allin');
    
    inq=inpolygon(xywq(:,1),xywq(:,2),...
        polygon_sides(:,1),polygon_sides(:,2));
    
    s=min(inq);
    if s== 0
        ss='inpolygon: ko';
    else
        ss='inpolygon: ok';
    end
    
    fprintf('\n \t >> '); disp(ss);
    xq=xywq(:,1); yq=xywq(:,2); wq=xywq(:,3);
    for ftype=1:5
        f=myfun(ftype);
        fxy=feval(f,xq,yq);
        J(domaintype,ftype)=wq'*fxy;
    end
end



fprintf('\n \n \t ----- BY TRIANGULATION (REFERENCE VALUE)');

II=zeros(length(domains),5);

for domaintype=1:length(domains)
    fprintf('\n \t * domaintype: %3.0f',domains(domaintype));
    polygon_sides=mydomain(domains(domaintype));
    xywr=polygauss(polygon_sides,2*degree,...
        'triangulation');
    x=xywr(:,1); y=xywr(:,2); w=xywr(:,3);
    for ftype=1:5
        f=myfun(ftype);
        fxy=feval(f,x,y);
        II(domaintype,ftype)=w'*fxy;
    end
end

%format long e
%[I' J' II']


fprintf('\n \n \t ----- FORMULA COMPARISON');

for domaintype=1:length(domains)
    DD=domains(domaintype);
    fprintf('\n \t * domaintype: %3.0f',DD);
    max_abserr=(max(abs(I(domaintype,:)-J(domaintype,:))));
    max_relerr=(max(abs(I(domaintype,:)-J(domaintype,:))./...
        abs(II(domaintype,:))));
    max_abserrI=(max(abs(I(domaintype,:)-II(domaintype,:))));
    max_abserrII=(max(abs(J(domaintype,:)-II(domaintype,:))));
    fprintf('\n \t max. abs. diff.: %2.2e',max_abserr);
    fprintf('\n \t max. abs. errs.: T: %2.2e Q: %2.2e',max_abserrI,...
        max_abserrII);
    fprintf('\n \t max. rel. err.: %2.2e',max_relerr);
end

if doplot == 1
    figure(1)
    M=size(polygon_sides,1); vv=[1:M 1];
    plot(polygon_sides(vv,1),polygon_sides(vv,2),...
        xt,yt,'r^');
    
    figure(2)
    M=size(polygon_sides,1); vv=[1:M 1];
    plot(polygon_sides(vv,1),polygon_sides(vv,2),...
        xq,yq,'rd');
end

fprintf('\n \n');