
function demo_polygauss_ade(degree,domain)

%--------------------------------------------------------------------------
% OBJECT.
%--------------------------------------------------------------------------
% This demo compares "polygauss.m" using "triangulation" option against
% the "quadrangulation" option, on several domain, testing the algebraic
% degree of exactness.
% Max absolute errors between these functions are reported.
% By setting the "domain" variable, several domain can be considered (see
% "mydomain.m" routine).
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% EXAMPLE.
%--------------------------------------------------------------------------
%
% Setting degree=20, domain=1,
%
% >> demo_polygauss
%
%  	 ----- BY TRIANGULATION
%  	 * domaintype:   1
%  	 [POLYGON]: 6 SIDES CONVEX POLYGON :    7 VERTICES
%
%  	 ----- BY QUADRANGULATION
%  	 * domaintype:   1
%  	 [POLYGON]: 6 SIDES CONVEX POLYGON :    7 VERTICES
%
%  	 ----- BY TRIANGULATION (REFERENCE VALUE)
%  	 * domaintype:   1
%  	 [POLYGON]: 6 SIDES CONVEX POLYGON :    7 VERTICES
%
%  	 ----- FORMULA COMPARISON
%  	 * domaintype:   1
%  	 max. abs. diff.: 5.04e-07
%  	 max. abs. errs.: T: 5.10e-07 Q: 7.45e-09
%  	 max. rel. err.: 1.35e-04
%
% >>
%
%--------------------------------------------------------------------------
% NOTE.
%--------------------------------------------------------------------------
%
% Here:
%
% "max. abs. diff." is the maximum absolute difference between the
% results achieved by the two methods on 5 tests functions.
%
% "max. abs. errs." is the maximum absolute difference between the
% results achieved by the two methods on 5 tests functions and a reference
% result by higher order rule. Notice that T stands for triang., Q for
% quadrang.
%
%--------------------------------------------------------------------------

if nargin < 1
    degree=30;
end

if nargin < 2
    domain=16;
end

%-------------- MAIN PROGRAM STARTS HERE ----------------------------------

doplot=1; % 0: no plot. 1: plot.
tol=10^(-13);

fprintf('\n \n \t ----- BY TRIANGULATION');


fprintf('\n \t * domaintype: %3.0f',domain);
polygon_sides=mydomain(domain);
rect=[min(polygon_sides(:,1)) max(polygon_sides(:,1)) ...
    min(polygon_sides(:,2)) max(polygon_sides(:,2))];
xywt=polygauss(polygon_sides,degree,...
    'triangulation');

int=inpolygon(xywt(:,1),xywt(:,2),...
    polygon_sides(:,1),polygon_sides(:,2));
s=min(int);
if s== 0
    ss='inpolygon: ko';
else
    ss='inpolygon: ok';
end


fprintf('\n \t >> '); disp(ss);
xt=xywt(:,1); yt=xywt(:,2); wt=xywt(:,3);
Vt = chebvand(degree,[xt yt],rect);
I=wt'*Vt;


fprintf('\n \n \t ----- BY QUADRANGULATION');


fprintf('\n \t * domaintype: %3.0f',domain);
polygon_sides=mydomain(domain);
rect=[min(polygon_sides(:,1)) max(polygon_sides(:,1)) ...
    min(polygon_sides(:,2)) max(polygon_sides(:,2))];

xywq=polygauss(polygon_sides,degree,...
    'quadrangulation_allin');

inq=inpolygon(xywq(:,1),xywq(:,2),...
    polygon_sides(:,1),polygon_sides(:,2));

s=min(inq);
if s== 0
    ss='inpolygon: ko';
else
    ss='inpolygon: ok';
end

fprintf('\n \t >> '); disp(ss);
xq=xywq(:,1); yq=xywq(:,2); wq=xywq(:,3);
Vq = chebvand(degree,[xq yq],rect);
J=wq'*Vq;




fprintf('\n \n \t ----- BY TRIANGULATION (REFERENCE VALUE)');

fprintf('\n \t * domaintype: %3.0f',domain);
polygon_sides=mydomain(domain);
rect=[min(polygon_sides(:,1)) max(polygon_sides(:,1)) ...
    min(polygon_sides(:,2)) max(polygon_sides(:,2))];
xywr=polygauss(polygon_sides,2*degree,...
    'triangulation');
x=xywr(:,1); y=xywr(:,2); w=xywr(:,3);
V = chebvand(degree,[x y],rect);
II=w'*V;

% [I' J' II' I'-II' J'-II']

fprintf('\n \n \t ----- FORMULA COMPARISON');

DD=domain;
fprintf('\n \t * domaintype: %3.0f',DD);

oki=find(abs(II)> tol);

max_abserrI=max(abs(I-II));
max_relerrI=max(abs((I(oki)-II(oki))./II(oki)));

max_abserrJ=max(abs(J-II));
max_relerrJ=max(abs((I(oki)-II(oki))./II(oki)));

fprintf('\n \t T: AE: %1.1e RE: %1.1e',max_abserrI,max_relerrI);
fprintf('\n \t Q: AE: %1.1e RE: %1.1e',max_abserrJ,max_relerrJ);


if doplot == 1
    figure(1)
    M=size(polygon_sides,1); vv=[1:M 1];
    plot(polygon_sides(vv,1),polygon_sides(vv,2),...
        xt,yt,'r^');
    
    figure(2)
    M=size(polygon_sides,1); vv=[1:M 1];
    plot(polygon_sides(vv,1),polygon_sides(vv,2),...
        xq,yq,'rd');
end

fprintf('\n \n');